
# q-expansion of newform 338.4.c.g, downloaded from the LMFDB on 28 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 338
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[171, [0, -1]]]
aps_data = [[2, -2], [1, -1], [-17, 0], [0, -35], [2, -2], [0, 0], [0, 19], [0, 94], [72, -72], [-246, 246], [100, 0], [-11, 11], [-280, 280], [0, -241], [-137, 0], [-232, 0], [0, -386], [0, -64], [-670, 670], [0, 55], [838, 0], [1016, 0], [-420, 0], [-934, 934], [0, -1154], [-292, 292], [52, 0], [-12, 12], [-1213, 0], [0, 1158], [540, -540], [-1571, 0], [0, -568], [0, -1845], [0, 2850], [-2763, 0], [-466, 0], [0, 2552], [-3768, 3768], [0, -3296], [-45, 45], [-364, 0], [0, -2482], [-1220, 1220], [-1593, 1593], [0, -1606], [-2469, 2469], [1943, -1943], [0, 3032], [-5311, 0], [-507, 0], [6795, 0], [0, -4442], [0, 5024], [-7065, 7065], [-1364, 1364], [0, 4304], [-1519, 1519], [0, -1996], [2918, 0], [-812, 812], [0, -1855], [-5350, 0], [2262, 0], [-1857, 0], [3870, 0], [0, 10520], [7839, 0], [0, 1275], [-375, 375], [1592, -1592], [2424, 0], [7970, -7970], [0, -12492], [-4284, 4284], [0, -2337], [3562, 0], [0, -5274], [4812, -4812], [0, -9448], [521, -521], [-7141, 0], [-6299, 6299], [0, -3231], [410, -410], [-9397, 0], [0, 654], [14982, -14982], [0, 10845], [-13072, 0], [-15340, 0], [-19615, 19615], [0, -10904], [18519, -18519], [14400, 0], [0, -20422], [15606, -15606], [1783, 0], [-11140, 11140], [-839, 0], [17279, 0], [-20577, 20577], [0, -11785], [-10887, 10887], [-11453, 0], [-14382, 0], [-10160, 10160], [-16842, 0], [-13830, 0], [15835, -15835], [0, 22894], [9886, -9886], [0, 8856], [-3764, 0], [0, 17775], [0, -13358], [0, -26062], [-20598, 20598], [-27104, 27104], [0, -7500], [10402, -10402], [19715, -19715], [-1168, 0], [0, 3800], [-17336, 17336], [-10088, 0], [0, 11730], [0, 2190], [-2470, 0], [8693, 0], [-14636, 14636], [27501, -27501], [-11888, 11888], [-10776, 10776], [0, -21758], [-8080, 8080], [0, 3981], [0, -26048], [0, 12434], [8113, -8113], [-732, 0], [-32559, 32559], [0, 43582], [23042, 0], [20690, -20690], [0, -43000], [33065, 0], [-28938, 0], [12508, 0], [19535, 0], [0, 18775], [43531, -43531], [6955, 0], [-40728, 40728], [-23653, 23653], [-206, 0], [0, -28352], [0, -17612], [12014, 0], [26737, 0], [-35566, 35566], [0, -12279], [-51523, 0], [0, -41027], [23562, -23562], [41307, 0], [26586, -26586], [0, -39298]]
